risk.bach.10 <- function(data, nyears){   
  
   bach.formula <- function(outcome){
    
      # age = continuous
      # female = binary indicator
      # cpd = cigarettes per day
      # smkyears = years smoked
      # qtyears = years quit
      # asb = asbestos exposure binary indicator
          
      #Formula from appendix of Bach's paper minus coefficients
      formula.string <- "female+
      asb+
      age+
      I(I((age-53.459001)^3)*I(age>53))+
      I(I((age-61.954825)^3)*I(age>61))+
      I(I((age-70.910335)^3)*I(age>70))+
      qtyears+
      I((qtyears)^3)+
      I(I((qtyears-0.50513347)^3)*I(qtyears>0))+
      I(I((qtyears-12.295688)^3)*I(qtyears>12))+
      smkyears+
      I(I((smkyears-27.6577)^3)*I(smkyears>27))+
      I(I((smkyears-40)^3)*I(smkyears>40))+
      I(I((smkyears-50.910335)^3)*I(smkyears>50))+
      cpd+
      I(I((cpd-15)^3)*I(cpd>15))+
      I(I((cpd-20.185718)^3)*I(cpd>20))+
      I(I((cpd-40)^3)*I(cpd>40))"
      				       
      formula(paste(outcome,"~",formula.string,collapse=""))
   }
   # Coefficients from diagnosis of lung cancer appendix
   bach.coef <- c(intercept =-9.7960571,
                female =-0.05827261,
                asb =0.2153936,
                age=0.070322812,
                age2=-0.00009382122,
                age3=0.00018282661,
                age4=-0.000089005389,
                qtyears=-0.085684793,
                qtyears2=0.0065499693,
                qtyears3=-0.0068305845,
                qtyears4=0.00028061519,
                smkyears=0.11425297,
                smkyears1=-0.000080091477,
                smkyears2=0.00017069483,
                smkyears3=-0.000090603358,
                cpd =0.060818386,
                cpd1 =-0.00014652216,
                cpd2 =0.00018486938,
                cpd3 =-0.000038347226)
   
   # Coefficients from probability of death from something other than lung cancer
   bach.coef.mort <- c(Intercept=-7.2036219, female=-0.49042298, asb=0.06084611, 
   						         age=0.099168033, age1=6.2174577e-06, age2=-1.2115774e-05, age3=5.8983164e-06, 
   						         qtyears=-0.023358962, qtyears2=0.0019208669, qtyears3=-0.0020031611, qtyears4=8.2294194e-05, 
   						         smkyears=0.020041889, smkyears1=6.5443781e-06, smkyears2=-1.3947696e-05, smkyears3=7.4033175e-06, 
   						         cpd=0.015490665, cpd1=-1.737645e-05, cpd2=2.1924149e-05, cpd3=-4.5476985e-06)

  vars <- c("female",
  				   "age",
  				   "cpd",
            "smkyears",
            "qtyears",
            "asb")

  var.not.in <- names(data)[!(names(data)%in%vars) & sapply(data, function(x) sum(is.na(x)))==0][1] # SELECT ARBITRARY VAR FOR OUTCOME
  
  P.lung <- matrix(0, nrow(data), ncol=10)
  P.death <- matrix(0, nrow(data), ncol=10)
  
  for(i in 1:10){
    # Calculate RRs and probabilities
    rr.lung  <- model.matrix(bach.formula(var.not.in), data)%*%bach.coef
    rr.death <- model.matrix(bach.formula(var.not.in), data)%*%bach.coef.mort
    P.lung[complete.cases(subset(data,select=vars)),i]  <- .99629^exp(rr.lung)
    P.death[complete.cases(subset(data,select=vars)),i] <- 0.9917663^exp(rr.death)
    # Update age, smoking and quit years for next cycle
    data$age      <- ifelse(is.na(data$age),NA,data$age+1)
    data$smkyears <- ifelse(is.na(data$smkyears),NA,data$smkyears+ifelse(data$qtyears==0,1,0)) # Add smoking year for current smokers
    data$qtyears  <- ifelse(is.na(data$qtyears),NA,data$qtyears+ifelse(data$qtyears==0,0,1)) # Add quit year for former smokers
   }
	
	# CUMULATIVE PROBABILITIES OF NO CANCER AND NO DEATH
   
   C.lung <- P.lung
   # Delete last column, put 1s in first column and shift other columns over by one
   C.lung[,2:10] <- C.lung[,1:9]
   C.lung[,1] <- 1
   C.lung <- t(apply(C.lung, 1, cumprod))
   
   C.mort <- P.death
   C.mort[,2:10] <- C.mort[,1:9]
   C.mort[,1] <- 1
   C.mort <- t(apply(C.mort, 1, cumprod))
	 
   #The final risk calculation
   #ifelse(rowSums((1-P.lung)*P.death*C.lung*C.mort)==0,NA,rowSums((1-P.lung)*P.death*C.lung*C.mort))
   tmp <- (1-P.lung)*P.death*C.lung*C.mort
   ret <- rowSums(tmp[, 1:nyears, drop=FALSE])
   tmp <- ret %in% 0
   if (any(tmp)) ret[tmp] <- NA

   ret
} 



risk.hoggart <- function(data){
   weibull.one.year <- function(t,lambda, gamma, HR=1){     
      shape <- exp(gamma)
      scale <- exp(lambda)
     
      #Time zero set to 35
      t1 <- t-35+1
      t0 <- t-35
      #Calculate HR times shape/scale difference for one time point
      diff <- HR*((t0/scale)^shape-(t1/scale)^shape)
      #Return exponentiated difference  
      exp(diff)
   }
   
   hoggart.coef <- function(current.age, age, former, smkyears, hr.lung, hr.mort){
      #For current smokers, cut points for age started smoking
      age.current <- c(-1,18,20,22,24,26,28,30,100)
      #For former smokers, cut points for age started smoking
      age.former <- c(-1,22,26,100)
      #For former smokers, cut points for duration of smoking
      duration <- c(-1,20,30,36,42,100)
      
      #Shape and scale parameters from Table 2 of Hoggart paper
      lambda.lung.current <- c(3.819, 4.056, 4.23, 4.339, 4.38, 4.567, 4.506, 4.504)
      gamma.lung.current <- c(0.999, 1.071, 1.298, 1.518, 1.679, 1.517, 1.615, 1.684)
      lambda.mort.current <- c(3.69, 3.774, 3.859, 3.944, 3.979, 4.049, 4.096, 4.069)
      gamma.mort.current <- c(1.22, 1.312, 1.56, 1.775, 1.925, 1.854, 1.838, 1.912)
      
      lambda.lung.former <- c(4.987, 4.723, 4.321, 5.179, 4.786, 4.651, 4.654, 4.366, 5.563, 4.68, 4.491, 4.241, 4.177)
      gamma.lung.former <- c(0.75, 0.819, 1.032, 1.165, 1.353, 1.46, 1.318, 1.662, 1.088, 1.705, 1.879, 2.336, 2.574)
      lambda.mort.former <- c(3.754, 3.75, 3.8, 4.008, 3.975, 3.954, 3.928, 3.982, 4.123, 4.058, 4.02, 3.999, 4.041)
      gamma.mort.former <- c(1.21, 1.511, 1.669, 1.621, 1.742, 1.835, 2.129, 2.15, 1.769, 1.865, 2.148, 2.36, 2.49)
      
      
      if(former==1){#For former smokers
      	strata.age <- as.numeric(cut(age, age.former, lab=1:3)) # <=22, 22-26, >26
      	strata.duration <- as.numeric(cut(smkyears, duration, lab=1:5)) # Note that strata.age=1 only sees cut points at 20&30
      	strata <- ifelse(strata.age==1,ifelse(strata.duration>=3,3,strata.duration),
      									ifelse(strata.age==2,strata.duration+3,strata.duration+8)) # ifelse(test, yes, no)
        #Now assign correct lambdas and gammas to people in each strata
      	lambda.lung <- lambda.lung.former[strata]
      	gamma.lung <- gamma.lung.former[strata]
      	lambda.mort <- lambda.mort.former[strata]
      	gamma.mort <- gamma.mort.former[strata]
      }
      else{#For current smokers
      	strata <- as.numeric(cut(age, age.current, lab=1:8)) #Cut points for age started smoking
        #Assign correct lambdas and gammas to people in each strata
      	lambda.lung <- lambda.lung.current[strata]
      	gamma.lung <- gamma.lung.current[strata]
      	lambda.mort <- lambda.mort.current[strata]
      	gamma.mort <- gamma.mort.current[strata]
      }
      	#Return weibull estimates for probability of getting lung cancer after one year
      (1-weibull.one.year(current.age,lambda=lambda.lung, gamma=gamma.lung,HR=hr.lung))*weibull.one.year(current.age,lambda=lambda.mort, gamma=gamma.mort,HR=hr.mort)
   }

	data$former <- 1*(data$qtyears>0)
	#Cigarettes per day = CPD
	lung.cpd.beta <- ifelse(data$former==0,0.105300795832441,0.0421816490792434) #This is kindof like log(HR), but only matches
	mort.cpd.beta <- ifelse(data$former==0,0.0497334248502077,0.0146138105962148) # 3 decimal points reliably
	#Representation of CPD in the model
	data$cpd <- ifelse(data$former==1, 
								ifelse(data$cpd<15, data$cpd-10.27171,15-10.27171), #former smokers
								ifelse(data$cpd<15, data$cpd-11.27674,15-11.27674) #current smokers
	)
	
	HR.lung <- exp(data$cpd*lung.cpd.beta)
	HR.mort <- exp(data$cpd*mort.cpd.beta)

	ifelse(complete.cases(subset(data,select=c("age","smoke.age.start","former","smkyears"))),
	       mapply(hoggart.coef, 
	              current.age=data$age[complete.cases(subset(data,select=c("age","smoke.age.start","former","smkyears")))], 
                age=data$smoke.age.start[complete.cases(subset(data,select=c("age","smoke.age.start","former","smkyears")))], 
                former=data$former[complete.cases(subset(data,select=c("age","smoke.age.start","former","smkyears")))], 
                smkyears = data$smkyears[complete.cases(subset(data,select=c("age","smoke.age.start","former","smkyears")))],
                hr.lung = HR.lung[complete.cases(subset(data,select=c("age","smoke.age.start","former","smkyears")))], 
	              hr.mort = HR.mort[complete.cases(subset(data,select=c("age","smoke.age.start","former","smkyears")))]),NA)
}



risk.llp <- function(data){
  llp.formula <- function(outcome){      
    # pneu = any diagnosis of pneumonia
    # asb = asbestos exposure binary indicator
    # prior.cancer = any prior cancer
    # fam.cancer.onset = 1 or more first degree ; early onset (3 categories)
    # smkyears.cat = 5 categories      
    formula.string <- "pneu+asb+prior.cancer+fam.cancer.onset+smkyears.cat"
    formula(paste(outcome,"~",formula.string,collapse=""))
  }
  
  llp.coef <- c(intercept =1,
                pneu=0.602,
                asb=0.634,
                prior.cancer=0.675,
                fam.cancer.onset1=0.703,
                fam.cancer.onset2=0.168,
                smkyears.cat1=0.769,
                smkyears.cat2=1.452,
                smkyears.cat3=2.507,
                smkyears.cat4=2.724)
  
  # To make the probabilities interpretable, they used Liverpool data to estimate the beta0, intercept here is from model
  llp.intercepts <- data.frame(
    incidence = c(15.5, 37.87, 88.65, 172.26, 329.02, 487.42, 616.45, 950.61, 1096.42,
                  5.97, 37.34, 68.14,175.24, 230.6, 288.06, 464.99, 594.19 ,497.09)/100000,
    intercept = -c(9.06,8.16, 7.31,6.63,5.97,5.56,5.31,4.83,4.68,
                   9.90,8.06,7.46,6.50,6.22,5.99,5.49,5.23,5.42),
    age = rep(1:9,2),
    female = rep(c(0,1),each=9))
  
  llp.intercept <- function(age, female){     
    #Assign age groups
    age.group <- function(age){
      if(age<45) 1
      else if(age<50) 2
      else if(age<55) 3
      else if(age<60) 4
      else if(age<65) 5
      else if(age<70) 6
      else if(age<75) 7
      else if(age<80) 8
      else 9
    }
    
    value <- function(age, age.cat, female){       
      if(age.cat==1)
        llp.intercepts$intercept[llp.intercepts$female==female&
                                   llp.intercepts$age==age.cat]
      else if(age.cat==2)
        sum(c(50-age-0.5, 0.5+age-45)*llp.intercepts$intercept[llp.intercepts$female==female&
                                                                 llp.intercepts$age>=2&llp.intercepts$age<=3])/5
      else if(age.cat==3)
        sum(c(55-age-0.5, 0.5+age-50)*llp.intercepts$intercept[llp.intercepts$female==female&
                                                                 llp.intercepts$age>=3&llp.intercepts$age<=4])/5
      else if(age.cat==4)
        sum(c(60-age-0.5, 0.5+age-55)*llp.intercepts$intercept[llp.intercepts$female==female&
                                                                 llp.intercepts$age>=4&llp.intercepts$age<=5])/5
      else if(age.cat==5)
        sum(c(65-age-0.5, 0.5+age-60)*llp.intercepts$intercept[llp.intercepts$female==female&
                                                                 llp.intercepts$age>=5&llp.intercepts$age<=6])/5
      else if(age.cat==6)
        sum(c(70-age-0.5, 0.5+age-65)*llp.intercepts$intercept[llp.intercepts$female==female&
                                                                 llp.intercepts$age>=6&llp.intercepts$age<=7])/5										
      else if(age.cat==7)
        sum(c(75-age-0.5, 0.5+age-70)*llp.intercepts$intercept[llp.intercepts$female==female&
                                                                 llp.intercepts$age>=7&llp.intercepts$age<=8])/5	
      else if(age.cat==8)
        sum(c(80-age-0.5, 0.5+age-75)*llp.intercepts$intercept[llp.intercepts$female==female&
                                                                 llp.intercepts$age>=8&llp.intercepts$age<=9])/5	
      else
        llp.intercepts$intercept[llp.intercepts$female==female&
                                   llp.intercepts$age==age.cat]
    }
    
    value(age, age.group(age), female)	
  }
  
  vars <- c("female",
            "age",            
            "asb",            
            "smkyears",            
            "pneu",            
            "prior.cancer",
            "fam.cancer.onset")
  
  var.not.in <- names(data)[!(names(data)%in%vars) & sapply(data, function(x) sum(is.na(x)))==0][1] # SELECT ARBITRARY VAR FOR OUTCOME
  expit <- function(x) exp(x)/(1+exp(x)) 
  data$smkyears.cat <- cut(data$smkyears, c(-1,1,21,41,60,max(data$smkyears,na.rm=TRUE)+1), right=FALSE, lab=1:5)
  #Manually construct the intercept
  intercept <- mapply(llp.intercept, age=data$age, female=data$female)
  X <- model.matrix(llp.formula(var.not.in), data)
  X[,1] <- intercept[complete.cases(subset(data,select= c("asb",            
                                                          "smkyears",            
                                                          "pneu",            
                                                          "prior.cancer",
                                                          "fam.cancer.onset")))]
  
  ifelse(complete.cases(subset(data,select=c("asb",            
                                             "smkyears",            
                                             "pneu",            
                                             "prior.cancer",
                                             "fam.cancer.onset"))),expit(X%*%llp.coef),NA)
} 



risk.llpi <- function(data){
   data$male <- as.numeric(data$female==0)
   #family history of lung cancer treated as late onset
   lp <- 0.036*data$age+0.391*data$male+0.043*data$smkyears+0.890*data$copd+1.044*data$prior.cancer+0.521*I(data$fam.cancer.onset==1)+0.071*I(data$fam.cancer.onset==2)

   #0.9728386 is the baseline survival at 8.7 years in the liverpool data
   #3.556 is the mean linear predictor in the liverpool data
   as.numeric(1-0.9728386**(exp(lp-3.556)))
}      



risk.spitz <- function(data){
   spitz.former.formula <- function(outcome){  
     # emp = emphysema (binary indicator)
     # dust = dust exposure  (binary indicator)
     # fam.cancer = 2 or more first degree relatives
     # fam.smoke.cancer = 1 or more first degree
     # no.hayfever = no hay fever (binary indicator)
     # age.stopped = categorical 3 groups	
     
     formula.string <- "emp+dust+fam.cancer+no.hayfever+age.stopped"   
     formula(paste(outcome,"~",formula.string,collapse=""))
   }
   
   spitz.current.formula <- function(outcome){
     # emp = emphysema (binary indicator)
     # dust = dust exposure  (binary indicator)
     # asb = asbestos exposure binary indicator
     # fam.smoke.cancer = 1 or more first degree
     # no.hayfever = no hay fever (binary indicator)
     # packyears.cat = categorical 4 groups	
     
     formula.string <- "emp+dust+asb+fam.smoke.cancer+no.hayfever+packyears.cat"  
     formula(paste(outcome,"~",formula.string,collapse=""))
   }
   
   spitz.baseline <- function(rr, age, female, former){
     age.group <- function(age){
       if(age<45) 1
       else if(age<50) 2
       else if(age<55) 3
       else if(age<60) 4
       else if(age<65) 5
       else if(age<70) 6
       else if(age<75) 7
       else if(age<80) 8
       else if(age<85) 9
       else 10
     }
     
     age.cat <- sapply(age,age.group)
     
     #I is incidence rate, h2 is mortality from something not lung cancer
     I <- spitz.rates$incidence[!is.na(spitz.rates$female)&!is.na(female)&spitz.rates$female==female&!is.na(spitz.rates$age)&!is.na(age.cat)&spitz.rates$age==age.cat] #This selects one gender and one agecat
     h2 <- spitz.rates$mortality[!is.na(spitz.rates$female)&!is.na(female)&spitz.rates$female==female&!is.na(spitz.rates$age)&!is.na(age.cat)&spitz.rates$age==age.cat]
     
     #Generate h1 (hazard from lung cancer) for men/women and former/current smokers
     h1 <- ifelse(is.na(female) | is.na(former),NA,
                  ifelse(female==1&former==1,I*3.76*(1-0.45352),
                         ifelse(female==1&former==0,I*4.17*(1-0.51404),
                                ifelse(female==0&former==1,I*3.17*(1-0.45352),I*3.88*(1-0.51404)))))
     #Model output
     h1*rr/(h1*rr+h2)*(1-exp(-(h1*rr+h2)))
   }
   
   spitz.former.coef <- c(intercept =0,
                          emp=0.9734,
                          dust = 0.4654,
                          fam.cancer = 0.4636,
                          no.hayfever = 0.3711,
                          age1 = 0.2130,
                          age2 = 0.4080)
   
   spitz.current.coef <- c(intercept =0,
                           emp=0.7561,
                           dust = 0.3067,
                           asb=0.4109,
                           fam.smoke.cancer = 0.3859,
                           no.hayfever = 0.4047,
                           py1 = 0.2219,
                           py2 = 0.3747,
                           py3 = 0.6151)
    
   spitz.rates <- data.frame(
     incidence = c(10.78, 25.49, 56.60, 116.58, 221.18, 346.77, 478.10, 564.36, 532.36, 498.44, 
                   11.03, 23.19, 45.51, 93.93, 164.9, 246.85, 318.69, 344.67, 308.28, 266.72)/100000,
     mortality = c(275.1, 400.7, 560.0, 786.9, 1210.2, 1855.1, 2947.4, 4836.4, 7980.7, 15559.4,
                   153.20, 218.80, 313.40, 479.10, 762.90, 1197.00, 1968.30, 3306.10, 5761.20, 14016.2)/100000,
     age = rep(1:10,2),
     female = rep(c(0,1),each=10))
     #Incidence and mortality rates are each done for women and men separately

   data$age.stopped[is.na(data$age.stopped)] <- 0
   data$former <- 1*(data$qtyears>0)   
   vars <- c("female", 
             "age",
             "asb",                  
             "emp",
             "dust",
             "former",
             "no.hayfever",
             "pkyr.cat",
             "fam.cancer",
             "fam.smoke.cancer",
             "age.stopped")
  
   var.not.in <- names(data)[!(names(data)%in%vars) & sapply(data, function(x) sum(is.na(x)))==0][1] # SELECT ARBITRARY VAR FOR OUTCOME

   data$packyears.cat <- cut(data$pkyr.cat,c(-1,28,42,57.5,max(data$pkyr.cat,na.rm=TRUE)+1),right=FALSE,labels=1:4)
   data$age.stopped <- cut(data$age.stopped, c(-1,42,53.1,max(data$age.stopped,na.rm=TRUE)+1),right=FALSE,labels=1:3)

   # DATA FRAME HAS TO HAVE VARIABLES FOR EACH MODEL
   rr.former <- ifelse(complete.cases(subset(data,select=c("emp","dust","fam.cancer","no.hayfever","age.stopped"))),model.matrix(spitz.former.formula(var.not.in), data)%*%spitz.former.coef,NA)
   rr.current <- ifelse(complete.cases(subset(data,select=c("emp","dust","asb","fam.smoke.cancer","no.hayfever","packyears.cat"))),model.matrix(spitz.current.formula(var.not.in), data)%*%spitz.current.coef,NA)
    
   rr.former <- exp(rr.former)
   rr.current <- exp(rr.current)

   risk.former <- ifelse(complete.cases(data$female)&complete.cases(data$former),
                         mapply(spitz.baseline,
                                rr=rr.former[complete.cases(data$female)&complete.cases(data$former)],
                                age=data$age[complete.cases(data$female)&complete.cases(data$former)],
                                female=data$female[complete.cases(data$female)&complete.cases(data$former)],
                                MoreArgs=list(former=1)),NA)
    
   risk.current <- ifelse(complete.cases(data$female)&complete.cases(data$former),
                          mapply(spitz.baseline,
                                rr=rr.current[complete.cases(data$female)&complete.cases(data$former)],
                                age=data$age[complete.cases(data$female)&complete.cases(data$former)],
                                female=data$female[complete.cases(data$female)&complete.cases(data$former)],
                                MoreArgs=list(former=0)),NA)

   ifelse(data$former==1,risk.former,risk.current)
} 



risk.tammemagi <- function(data){
   tammemagi.formula <- function(outcome){   
       # age = continuous
       # race = categorical
       # edu = ordinal
       # bmi = continuous
       # fam.lung.cancer = binary
       # prior.cancer = binary
       # copd = binary
       # cpd = cigarettes per day
       # current = binary
       # smkyears = years smoked
       # qtyears = years quit
       formula.string <- "I(age-62)+
                    black+hispanic+asian+indian+islander+
                    I(edu6-4)+
                    I(bmi-27)+
                    fam.lung.cancer+
                    prior.cancer+
                    copd+
                    current+
                    I(10/cpd-.402154)+
                    I(qtyears-10)+
                    I(smkyears-27)"              
       
       formula(paste(outcome,"~",formula.string,collapse=""))
   }

   tammemagi.coef <- c(intercept=-4.532506,
                       age = 0.0778868,
                       black = 0.3944778,
                       hispanic = -0.7434744,
                       asian = -0.466585,
                       indian = 0,
                       islander = 1.027152,
                       edu = -0.0812744,
                       bmi = -0.0274194,
                       fam.lung.cancer = 0.587185,
                       prior.cancer = 0.4589971,
                       copd = 0.3553063,
                       current = 0.2597431,
                       cpd = -1.822606,
                       qtyears = -0.0308572,
                       smkyears = 0.0317321)
              
   data$black <- 1*(data$race==1)
   data$hispanic <- 1*(data$race==2)
   data$fam.lung.cancer <- 1*(data$fam.lung.trend>0)
   data$current <- 1*(data$qtyears==0)
   vars <- c("age",
             "black","hispanic","asian","islander","indian",
             "edu6",
             "bmi",
             "fam.lung.cancer",
             "prior.cancer",
             "copd",
             "cpd",
             "current",
             "smkyears",
             "qtyears")
  
   var.not.in <- names(data)[!(names(data)%in%vars) & sapply(data, function(x) sum(is.na(x)))==0][1] # SELECT ARBITRARY VAR FOR OUTCOME
 
   X <- model.matrix(tammemagi.formula(var.not.in), data)

   lp <- ifelse(complete.cases(subset(data,select=vars)),X%*%tammemagi.coef,NA)

   exp(lp)/(1+exp(lp))
} 



risk.kovalchik <- function (begin, end, newdata, coxph1, ...) {

    # Default value for end is 5
    if (!length(end)) end <- 5      

    c.coxph.risk <- function (coxph, ...) 
    {
      models <- list(coxph)
      if (!missing(..1)) 
        models <- c(models, list(...))
      models
    }  
  
    coxph.relrisk.uncentered <- function (coxph.object, newdata) 
    {
      center <- coxph.object$means %*% coef(coxph.object)
      if (!missing(newdata)) 
        lp <- predict(coxph.object, newdata, type = "lp")
      else lp <- coxph.object$linear.predictor
      exp(lp + rep(center,length(lp)))
    }  

    projection.relrisk <- function (object, data) 
    {
      if (is.numeric(object)) 
        return(object)
      else if (class(object) == "coxph") 
        if (missing(data)) 
          return(coxph.relrisk.uncentered(object))
        else return(coxph.relrisk.uncentered(object, data))
      else stop(cat("No method for class", class(object)))
    }    
    
    #calculate absolute risk given hazards/survival and relative risks
    risk.fixed.interval <- function(H, RR) {
      if (!is.list(H)) 
        0
      else {
        absrisk <- H[[1]]$surv^RR[1] * H[[1]]$haz * RR[1]
        for (i in 2:length(H)) {
          absrisk <- absrisk * (H[[i]]$surv^RR[i])
        }
        sum(absrisk)
      }
    }
    
    models <- c.coxph.risk(coxph1, ...)

    #calculate relative risk for each subject and store as list
    rr <- sapply(models, projection.relrisk, data = newdata)

    if (is.matrix(rr)) { rr.list <- lapply(1:nrow(rr), function(x) rr[x, ])
    } else { rr.list <- list(rr)}

    #estimate risk
    if (length(begin) == 1) {    
       AllVars <- unique(unlist(sapply(models, function(x) all.vars(x$formula))))
        in.interval <- function(x, begin, end) x >= begin & x <= end
        if ("lung.cancer.death" %in% AllVars){
          H <- list(models[[1]]$basehaz[in.interval(models[[1]]$basehaz$time, begin, end),], models[[2]]$basehaz_LCDRAT[in.interval(models[[1]]$basehaz$time, begin, end),])
        } else if ("case" %in% AllVars){
          H <- list(models[[1]]$basehaz[in.interval(models[[1]]$basehaz$time, begin, end),], models[[2]]$basehaz_LCRAT[in.interval(models[[1]]$basehaz$time, begin, end),])        
        }
        #calculate absolute risk given hazards/survival for average covariate values and relative risks
        risks <- mapply(risk.fixed.interval, RR = rr.list, MoreArgs = list(H = H))
    } else {
        stop("No code for length(begin) != 1")
        #risks <- mapply(risk, begin = begin, end = end, RR = rr.list, MoreArgs = list(models = models))
    }

    ifelse(complete.cases(newdata),risks,NA)
}

risk.pittsburgh <- function(data){
  S <- -10*I(data$smkyears<30)+8*I(data$smkyears>=40&data$smkyears<50)+14*I(data$smkyears>=50)+
    4*I(data$smkyears<30&data$age>=57)+4*I(data$smkyears>=30&data$smkyears<40&data$age>=59)+
    4*I(data$smkyears>=40&data$smkyears<50&data$age>=61)+4*I(data$smkyears>=50&data$age>=68)-
    3*I(data$qtyears>=1)-4*I(data$cpd<20)+2*I(data$cpd>=30&data$cpd<40)+5*I(data$cpd>=40)  
  
  as.numeric(1/(1+exp(-1*(-4.2195+0.10*S))))
}      


#' Wrapper to the cut() function
#'
#' A wrapper to the cut() function, so that you can automatically break into quantiles as
#' the default behavior, otherwise if the breakpoints are included, then just break on those.
#' In all cases, include.lowest is set to True      
#' @export
cuts <- function(data,npieces,simple.labels=TRUE,...) {

  if (length(npieces)==0 | any(is.na(npieces)) | any(!is.numeric(npieces)))
    stop("npieces must be a numeric scalar or a vector of breakpoints")
  
  if (length(npieces)==1)
    # Just break into quantiles.  Use quantile labelling: Q1,Q2,Q3,etc. if you want
    if (simple.labels)
      cutdata <- cut(data,breaks=quantile(data,seq(0,1,1/npieces),na.rm=T,...),
                     include.lowest=T,labels=paste("Q",1:npieces,sep=""),...)
    else
      cutdata <- cut(data,breaks=quantile(data,seq(0,1,1/npieces),na.rm=T,...),
                     include.lowest=T,...)
  else
    # Break into pieces as specified by the breakpoints
    if (simple.labels)
      cutdata <- cut(data,breaks=npieces,include.lowest=T,
                     labels=paste("Q",1:(length(npieces)-1),sep=""),...)
    else
      cutdata <- cut(data,breaks=npieces,include.lowest=T,...)
  
  return(cutdata)
}



#' Risk Predictions from Lung Cancer Models
#'
#' The R package provides individual risks of lung cancer and lung cancer death 
#' based on various published papers: Bach et al., 2003; Spitz et al., 2007; 
#' Cassidy et al., 2008 (LLP); Hoggart et al., 2012; Tammemagi et al., 2013 (PLCOm2012); 
#' Marcus et al., 2015 (LLPi); Wilson and Weissfeld, 2015 (Pittsburgh);
#' Katki et al., 2016 (LCRAT, LCDRAT, and versions constrained to a few predictive variables);
#' Katki et al., 2018.
#' This package also estimates the Life Years Gained From Screening-CT (LYFS-CT) 
#' as per Cheung et al., 2019. It requires the same variables as LCDRAT plus
#' 12 additional comorbidities and the year of patient assessment.
#'
#' @section Warning:
#'  VGAM and survey are required dependency of this package.
#'  VGAM and survey may automatically be installed the first time this package is used.
#'  Inputs must be in numerical format to ensure correct output.
#'  For data frame x, this can be checked using sapply(x,class)
#'
#' @param x A data frame or matrix containing individuals' covariate values.
#'  Columns 22-33 are needed only to estimate life gained from undergoing CT screening.
#'  Covariates should be in the following columns and numerical formats:
#'
#'  \itemize{
#'  \item column 1 - current age (numeric);
#'  \item column 2 - gender (1=Female, 0=Male);
#'  \item column 3 - years smoked (numeric);
#'  \item column 4 - years quit (numeric with 0 to indicate current smoker);
#'  \item column 5 - cigarettes per day (numeric);
#'  \item column 6 - race (0=Non-hispanic white,
#'                   1=Non-hispanic Black/African American, 
#'                   2=Hispanic, 
#'                   3=Other Ethnicity);
#'  \item column 7 - lung disease (1=COPD or Emphysema, 0=No COPD or Emphysema);
#'  \item column 8 - number of first degree relatives with lung cancer (0,1,2);
#'  \item column 9 - bmi;
#'  \item column 10 - highest education level (1=<12 grade, 
#'                                       2=HS graduate, 
#'                                       3=post hs, no college, 
#'                                       4=associate degree/some college, 
#'                                       5=bachelors degree, 
#'                                       6=graduate school);
#'  \item column 11 - asbestos exposure (1=Yes,0=No);
#'  \item column 12 - prior history of pneumonia (1=Yes,0=No);
#'  \item column 13 - prior history of cancer (1=Yes,0=No);
#'  \item column 14 - family history of lung cancer (0=none, 1=early onset, 2=late onset);
#'  \item column 15 - Dust exposure  (1=Yes,0=No);
#'  \item column 16 - 2 or more first degree relatives with cancer (binary indicator);
#'  \item column 17 - 1 or more first degree relatives with smoking cancer (binary indicator);
#'  \item column 18 - no hay fever (1=No Hay Fever,0=Yes Hay Fever);
#'  \item column 19 - asian ethnicity (1=Yes,0=No);
#'  \item column 20 - islander ethnicity (1=Yes,0=No));
#'  \item column 21 - American indian ethnicity (1=Yes,0=No);
#'  \item column 22 - Hypertension (1=Yes,0=No);
#'  \item column 23 - Coronary Heart Disease (1=Yes,0=No);
#'  \item column 24 - Angina pectoris (1=Yes,0=No);
#'  \item column 25 - Heart Attack (1=Yes,0=No);
#'  \item column 26 - Other heart disease (1=Yes,0=No);
#'  \item column 27 - Stroke (1=Yes,0=No);
#'  \item column 28 - Diabetes (1=Yes,0=No);
#'  \item column 29 - Chronic bronchitis in past year (1=Yes,0=No);
#'  \item column 30 - Weak/failing kidneys in past year (1=Yes,0=No);
#'  \item column 31 - Liver condition in past year (1=Yes,0=No);
#'  \item column 32 - Health problem requiring special eqiupment (1=Yes,0=No);
#'  \item column 33 - Year of assessment.
#'  }
#'  
#'  Risk factors used by each model
#'  \tabular{cccccccccccc}{
#'  Covariate \tab Bach \tab Spitz \tab LLP \tab Hoggart \tab PLCOm2012 \tab Pittsburgh \tab LLPi \tab LCRAT \tab LCDRAT \tab constrainedLCRAT \tab constrainedLCDRAT\cr
#'  Age \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \cr
#'  Gender \tab Yes \tab Yes \tab Yes \tab No \tab No \tab No \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \cr
#'  Race/ethnicity \tab No \tab No \tab No \tab No \tab Yes \tab No \tab No \tab Yes \tab Yes \tab No \tab No \cr
#'  Asian \tab No \tab No \tab No \tab No \tab Yes \tab No \tab No \tab No \tab No \tab No \tab No \cr
#'  Pacific islander \tab No \tab No \tab No \tab No \tab Yes \tab No \tab No \tab No \tab No \tab No \tab No \cr
#'  America Indian \tab No \tab No \tab No \tab No \tab Yes \tab No \tab No \tab No \tab No \tab No \tab No \cr
#'  Education \tab No \tab No \tab No \tab No \tab Yes \tab No \tab No \tab Yes \tab Yes \tab No \tab No \cr 
#'  BMI \tab No \tab No \tab No \tab No \tab Yes \tab No \tab No \tab Yes \tab Yes \tab No \tab No \cr
#'  Smoking status \tab No \tab Yes \tab No \tab Yes \tab Yes \tab Yes \tab No \tab No \tab No \tab No \tab No \cr
#'  Years/Age quit \tab Yes \tab Yes \tab No \tab Yes \tab Yes \tab No \tab No \tab Yes \tab Yes \tab Yes \tab Yes \cr
#'  Years smoked \tab Yes \tab No \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \tab Yes \cr
#'  Cigs per day \tab Yes \tab No \tab No \tab Yes \tab Yes \tab Yes \tab No \tab Yes \tab Yes \tab Yes \tab Yes \cr
#'  Pack-years \tab No \tab Yes \tab No \tab No \tab No \tab No \tab No \tab Yes \tab Yes \tab Yes \tab Yes \cr
#'  Prior cancer \tab No \tab No \tab Yes \tab No \tab Yes \tab No \tab Yes \tab No \tab No \tab No \tab No \cr
#'  Lung disease \tab No \tab Yes \tab No \tab No \tab Yes \tab No \tab Yes \tab Yes \tab Yes \tab No \tab No \cr
#'  Pneumonia \tab No \tab No \tab Yes \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab No \cr
#'  Hayfever \tab No \tab Yes \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab No \cr
#'  Asbestos exposure \tab Yes \tab Yes \tab Yes \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab No \cr
#'  Dust exposure \tab No \tab Yes \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab No \cr
#'  Any FDR w/ cancer \tab No \tab Yes \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab No \cr
#'  Any FDR w/ smoking-related cancer \tab No \tab Yes \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab No \cr
#'  Any FDR w/ LC \tab No \tab No \tab Yes \tab No \tab Yes \tab No \tab Yes \tab Yes \tab Yes \tab No \tab No \cr
#'  Num. FDR w/ LC \tab No \tab No \tab No \tab No \tab No \tab No \tab No \tab Yes \tab Yes \tab No \tab No \cr
#'  Early onset FDR w/ LC \tab No \tab No \tab Yes \tab No \tab No \tab No \tab Yes \tab No \tab No \tab No \tab No
#'  }
#' @param impute.missing Option to impute missing variables using the 
#'        NHIS 2015 survey (default=TRUE). 
#' @param counterfactual.race 0-3 to compute countefactual estimates,
#'            where (0=Non-hispanic white,
#'                   1=Non-hispanic Black/African American, 
#'                   2=Hispanic, 
#'                   3=Other Ethnicity).
#'           The default is 0.
#'
#' @return A numeric matrix containing individuals' predictions:
#' 
#'  \itemize{
#'  \item column 1 - An indicator variable for whether the individual is eligible 
#'              for CT lung screening according to 
#'              US Preventive Services Task Force (USPSTF) recommendations.
#'  \item column 2 - This is the probability of dying from lung cancer 
#'              within 5 years if not undergoing screening (LCDRAT from Katki, 2016).
#'  \item column 3 - This is the reduction in the probability of dying from lung cancer in 5 years 
#;              if undergoing 3 yearly CT lung screens as in the NLST (Katki, 2016).
#'  \item column 4 - This is the probability of being diagnosed with lung cancer
#'              within 5 years if not undergoing screening (LCRAT from Katki, 2016).
#'  \item column 5 - This is the extra probability of lung cancer diagnosis in 5 years
#'              if undergoing 3 yearly CT lung screens as in the NLST  (Katki, 2016).
#'  \item column 6 - This is the probability of having at least one false-positive 
#'              CT screen out of 3 screens (Katki, 2016).
#'  \item column 7 - This is the expected number of false-positive CT screens 
#'               after 3 screens (Katki, 2016).
#'  \item column This is the probability of being diagnosed with lung cancer 
#'              within 10 years if not undergoing screening (Bach, 2003).
#'  \item column 9 - This is the probability of being diagnosed with lung cancer 
#'              within 1 years if not undergoing screening (Hoggart, 2012).
#'  \item column 10 - This is the probability of being diagnosed with lung cancer 
#'              within 5 years if not undergoing screening (LLP, 2008).
#'  \item column 11 - This is the probability of being diagnosed with lung cancer 
#'              within 8.7 years if not undergoing screening (LLPi, 2015).
#'  \item column 12 - This is the probability of being diagnosed with lung cancer 
#'              within 1 years if not undergoing screening (Spitz, 2007).
#'  \item column 13 - This is the probability of being diagnosed with lung cancer 
#'              within 6 years if not undergoing screening (PLCOm2012 from Tammemagi, 2013).
#'  \item column 14 - This is the probability of being diagnosed with lung cancer
#'              within 6 years if not undergoing screening (Pittsburgh, 2015).
#'  \item column 15 - This is the probability of dying from lung cancer 
#'              within 5 years if not undergoing screening (constrained LCDRAT).
#'  \item column 16 - This is the probability of being diagnosed with lung cancer 
#'              within 5 years if not undergoing screening (constrained LCRAT).
#'  \item column 17 - Days of life gained from undergoing 3 rounds of CT screening 
#'              (LYFS-CT from Cheung, 2019).
#'  \item column 18 - Life expectancy without CT screening.
#'  \item column 19 - Life expectancy with CT screening.
#'  \item column 20 - Years of life gained if lung cancer is found early due to screening.
#'  \item column 21 - Years of life gained if lung cancer death is averted due to screening
#'                          
#'  }
#'  
#' @author Li C. Cheung, \email{li.cheung@nih.gov}, Stephanie A. Kovalchik, Hormuzd A. Katki
#'
#' @references 
#' \itemize{
#'  \item Bach PB, Kattan MW, Thornquist MD, et al. 
#'        Variations in lung cancer risk among smokers. J Natl Cancer Inst 2003;95:470-8.
#'  \item Spitz MR, Hong WK, Amos CI, et al. 
#'        A risk model for prediction of lung cancer. J Natl Cancer Inst 2007;99:715-26.
#'  \item Cassidy A, Myles JP, van Tongeren M, et al. 
#'        The LLP risk model: an individual risk prediction model for lung cancer. 
#'        Br J Cancer 2008;98:270-6.
#'  \item Hoggart C, Brennan P, Tjonneland A, et al. 
#'        A risk model for lung cancer incidence. Cancer Prev Res (Phila) 2012;5:834-46.
#'  \item Tammemagi MC, Katki HA, Hocking WG, et al. 
#'        Selection criteria for lung-cancer screening. N Engl J Med 2013;368:728-36.
#'  \item Marcus MW, Chen Y, Raji OY, Duffy SW, Field JK.
#'        LLPi: Liverpool lung project risk prediction model for lung cancer incidence.
#'        Cancer Prev Res (Phila) 2015;8:570-5.
#'  \item Wilson DO, Weissfeld J.
#'        A simple model for predicting lung cancer occurrence in a lung cancer screening program:
#'        the Pittsburgh predictor. Lung Cancer 2015;89:31-37.      
#'  \item Katki HA, Kovalchik SA, Berg CD, Cheung LC, Chaturvedi AK.
#'        Development and validation of risk models to select ever-smokers
#'        for CT lung cancer screening. JAMA. 2016;315:2300-11.
#'        doi: 10.1001/jama.2016.6255.
#'  \item Katki HA, Kovalchik SA, Petito LC, Cheung LC, Jacobs E, Jemal A, Berg CD, Chaturvedi AK.
#'        Implications of nine risk prediction models for selecting ever-smokers for computed
#'        tomography lung cancer screening.  Ann Intern Med. 2018;169(1):10-19. doi: 10.7326/M17-2701. 
#'  \item Cheung LC, Berg CD, Castle PE, Katki HA, Chaturvedi AK.
#'        Life-gained-based versus risk-based selection of smokers for lung cancer screening.
#'        Ann Intern Med. 2019;171:623-632. doi:107326/M19-1263.      
#'           
#' }
#'             
#'
#' @export
#' @examples
#' age <- c(66,58,75,72,56)
#' bmi <- c(23,28,26,27,24)
#' cpd <- c(36,36,40,24,40)
#' emp <- c(0,1,1,0,1)
#' fam.lung.trend <- c(0,2,0,2,0)
#' female <- c(0,1,0,1,0)
#' smkyears <- c(43,37,45,42,29)
#' qtyears <- c(0,0,9,6,6)
#' race <- c(0,1,2,2,3)
#' edu6 <- c(3,5,4,5,5)
#' asb <- c(0,0,0,0,0)
#' pneu <- c(0,0,0,0,0)
#' prior.cancer <- c(0,0,0,0,0)
#' fam.cancer.onset <- c(0,1,0,2,0)
#' dust <- c(0,0,0,0,0)
#' fam.cancer <- c(0,1,0,1,0)
#' fam.smoke.cancer <- c(0,1,0,1,0)
#' no.hayfever <- c(1,1,1,1,1)
#' asian <- c(0,0,0,0,1)
#' islander <- c(0,0,0,0,0)
#' indian <- c(0,0,0,0,0) 
#' hypertension <- c(0,0,1,0,1)
#' chd <- c(0,0,0,0,0)
#' angina <- c(0,0,0,0,0)
#' heartattack <- c(0,0,0,0,1)
#' heartdisease <- c(0,0,0,0,0)
#' stroke <- c(0,0,0,0,0)
#' diab <- c(1,0,0,0,0)
#' bron <- c(0,1,0,0,1)
#' kidney <- c(0,0,0,0,0)
#' liver <- c(0,0,0,0,0)
#' speceq <- c(0,1,0,0,0)
#' year <- rep(2019,5)
#'
#' persons <- data.frame(age,
#'                      female,
#'                      smkyears,
#'                      qtyears,
#'                      cpd,
#'                      race,
#'                      emp,
#'                      fam.lung.trend,
#'                      bmi,
#'                      edu6,
#'                      asb,
#'                      pneu,
#'                      prior.cancer,
#'                      fam.cancer.onset,
#'                      dust,
#'                      fam.cancer,
#'                      fam.smoke.cancer,
#'                      no.hayfever,
#'                      asian,
#'                      islander,
#'                      indian,
#'                      hypertension,
#'                      chd,
#'                      angina,
#'                      heartattack,
#'                      heartdisease,
#'                      stroke,
#'                      diab,
#'                      bron,
#'                      kidney,
#'                      liver,
#'                      speceq,
#'                      year)
#' 
#' persons_predictions <- lcmodels(persons)
#' persons_predictions

lcmodels <- function(x, impute.missing=TRUE, counterfactual.race=0, nyears=NULL, nyears.mortality=5) {

  ##########################################################################################################
  # History
  # 2021-08-13 Make changes to line: survx <- (exp(-1*haz[which(haz$time>age[x]),1])/exp(-1*haz[which(haz$time==age[x]),1]))^rs[x]
  # Email: "We should change that code to use the time just preceding age[x] if 
  #  the hazard at age[x] is not defined.  Was age 41 the only integer age affected from ages 40-84 
  #  (for 40, we should remove the denominator altogether)? Best, Li
  # 2021:09-03 Make changes to age[x] based on email:
  #   We encountered some errors with the LYG calculations.  
  #   Currently, z subsets to non-missing rows, and we sapply the lyg function to z.  
  #   However, within the lyg function, we index age[x], but the age vector contains both non-missing and missing rows.  
  #   Therefore non-missing rows after the missing rows are reading the incorrect age[x].
  # 2021-10-13 Add option for counterfactual race. Email:
  #   For lcrisks and lcmodels, we want to create a counterfactual set of estimates for the life expectancy and LYG calculations.
  #   These counterfactual estimates will use:
  #   1) the original lung cancer death risk predictions with the individuals original race
  #   2) comorbidity imputations under the individuals counterfactual race (set default to white)
  #   3) the mortality model predictions under the individuals counterfactual race (set default to white)
  # 2023-06-06 Add option nyears. Currently, this option is only for Bach model, LCRAT, LCDRAT.
  # 2023-06-30 Add option nyears.mortality. 
  ##########################################################################################################

  checkLogical(impute.missing)  
  checkCounterfactual(counterfactual.race) 
  x <- checkInputData(x)
  if (length(nyears)) checkInt(nyears, "nyears", min=1, max=10) 
  checkNumeric(nyears.mortality, maxval=98)

  # First call using no counterfactual race
  obj0 <- lcmodels.main(x, impute.missing=impute.missing, counterfactual.race=NULL, 
                        nyears=nyears, nyears.mortality=nyears.mortality)

  # Second call using counterfactual race
  obj1 <- lcmodels.main(x, impute.missing=impute.missing, counterfactual.race=counterfactual.race,
                        nyears=nyears,  nyears.mortality=nyears.mortality)

  ret  <- mergeObj.counterfactual(obj0, obj1) 
  ret

}

lcmodels.main <- function(x, impute.missing=TRUE, counterfactual.race=0, nyears=NULL, nyears.mortality=5) {

  flag1       <- ncol(x) >= 33
  counterFlag <- !is.null(counterfactual.race)

  # To make backwards compatible, nyears should be 10 for Bach model and 5 years for other models
  nyears.bach <- nyears
  if (!length(nyears.bach)) nyears.bach <- 10
  if (!length(nyears)) nyears <- 5

  age <- ifelse(is.na(x[,1])==0 & x[,1]>=0,x[,1],NA)
  female <- ifelse(is.na(x[,2])==0 & x[,2] %in% c(0,1),x[,2],NA)
  smkyears <- ifelse(is.na(x[,3])==0 & x[,3]>=0,x[,3],NA)
  qtyears <- ifelse(is.na(x[,4])==0 & x[,4]>=0,x[,4],NA)
  cpd <- ifelse(is.na(x[,5])==0 & x[,5]>=0,x[,5],NA)
  race <- as.factor(ifelse(is.na(x[,6])==0 & x[,6] %in% c(0,1,2,3),x[,6],NA))
  emp <- ifelse(is.na(x[,7])==0 & x[,7] %in% c(0,1),x[,7],NA)
  fam.lung.trend <- ifelse(is.na(x[,8])==0 & x[,8] %in% c(0,1,2),x[,8],NA)
  bmi <- ifelse(is.na(x[,9])==0 & x[,9]>0,x[,9],NA)
  edu6 <- ifelse(is.na(x[,10])==0 & x[,10] %in% c(1,2,3,4,5,6),x[,10],NA)
  pkyr.cat <- smkyears*cpd/20

  race.orig      <- race
  if (counterFlag) {
    race.counter <- as.factor(rep(counterfactual.race, length(race)))
  } else {
    race.counter <- race 
  }

  # Imputation uses the counterfactual race or the original race (if counterfactual.race = NULL)
  race <- race.counter
  if (impute.missing==TRUE){
     bmi[is.na(bmi)==1] <- 29.994-0.1115*age[is.na(bmi)==1]+1.7528*I(race[is.na(bmi)==1]==1)+
       0.9577*I(race[is.na(bmi)==1]==2)-1.6921*I(race[is.na(bmi)==1]==3)+0.1091*female[is.na(bmi)==1]+
       0.0531*cpd[is.na(bmi)==1]-0.1608*log(pkyr.cat[is.na(bmi)==1])+0.9115*log(smkyears[is.na(bmi)==1])+
       0.9508*log(qtyears[is.na(bmi)==1]+1)
     bmi <- checkImputed(bmi, "bmi", warn=1)
     edu6[is.na(edu6)==1] <- 3.6862+0.1192*log(age[is.na(edu6)==1])-0.4907*I(race[is.na(edu6)==1]==1)-
       0.9958*I(race[is.na(edu6)==1]==2)-0.0708*I(race[is.na(edu6)==1]==3)+0.0823*female[is.na(edu6)==1]-
       0.2484*I(cpd[is.na(edu6)==1]>=20)-0.0434*log(pkyr.cat[is.na(edu6)==1])-0.0207*smkyears[is.na(edu6)==1]+
       0.0964*log(qtyears[is.na(edu6)==1]+1)
     edu6 <- checkImputed(edu6, "education", warn=1)
     lp <- -5.2038+1.1218*log(age[is.na(fam.lung.trend)==1])-1.022*I(race[is.na(fam.lung.trend)==1]==1)-
           1.4558*I(race[is.na(fam.lung.trend)==1]==2)-1.3052*I(race[is.na(fam.lung.trend)==1]==3)+
           0.0917*female[is.na(fam.lung.trend)==1]-0.3626*log(cpd[is.na(fam.lung.trend)==1])+
           0.4853*log(pkyr.cat[is.na(fam.lung.trend)==1])-0.3273*sqrt(smkyears[is.na(fam.lung.trend)==1])-
           0.0184*qtyears[is.na(fam.lung.trend)==1]
     lp <- checkImputed(lp, "number of parents with lung cancer", warn=1)
     fam.lung.trend[is.na(fam.lung.trend)==1] <- exp(lp)/(1+exp(lp))
     lp <- -9.2854+0.1946*sqrt(age[is.na(emp)==1])-0.4025*I(race[is.na(emp)==1]==1)-1.139*I(race[is.na(emp)==1]==2)-
           1.0095*I(race[is.na(emp)==1]==3)-0.0142*female[is.na(emp)==1]+0.1818*I(cpd[is.na(emp)==1] >= 20)+
           0.0125*pkyr.cat[is.na(emp)==1]+0.7645*sqrt(smkyears[is.na(emp)==1])+0.0198*sqrt(qtyears[is.na(emp)==1])
     lp <- checkImputed(lp, "lung disease", warn=1)
     emp[is.na(emp)==1] <- exp(lp)/(1+exp(lp))
   }

  age.stopped <- ifelse(is.na(age-qtyears)==0 & age-qtyears>=0,age-qtyears,NA)
  smoke.age.start <- ifelse(is.na(age-smkyears-qtyears)==0 & age-smkyears-qtyears>=0,age-smkyears-qtyears,NA)
  asb <- ifelse(is.na(x[,11])==0 & x[,11] %in% c(0,1),x[,11],NA)
  pneu <- ifelse(is.na(x[,12])==0 & x[,12] %in% c(0,1),x[,12],NA)
  prior.cancer <- ifelse(is.na(x[,13])==0 & x[,13] %in% c(0,1),x[,13],NA)
  fam.cancer.onset <- as.factor(ifelse(is.na(x[,14])==0 & x[,14] %in% c(0,1,2),x[,14],NA))
  copd <- emp
  dust <- ifelse(is.na(x[,15])==0 & x[,15] %in% c(0,1),x[,15],NA)
  fam.cancer <- ifelse(is.na(x[,16])==0 & x[,16] %in% c(0,1),x[,16],NA)
  fam.smoke.cancer <- ifelse(is.na(x[,17])==0 & x[,17] %in% c(0,1),x[,17],NA)
  no.hayfever <- ifelse(is.na(x[,18])==0 & x[,18] %in% c(0,1),x[,18],NA)
  asian <- ifelse(is.na(x[,19])==0 & x[,19] %in% c(0,1),x[,19],NA)
  islander <- ifelse(is.na(x[,20])==0 & x[,20] %in% c(0,1),x[,20],NA)
  indian <- ifelse(is.na(x[,21])==0 & x[,21] %in% c(0,1),x[,21],NA)
  if (flag1){
    hypertension <- ifelse(is.na(x[,22])==0 & x[,22] %in% c(0,1),x[,22],NA)
    chd <- ifelse(is.na(x[,23])==0 & x[,23] %in% c(0,1),x[,23],NA)
    angina <- ifelse(is.na(x[,24])==0 & x[,24] %in% c(0,1),x[,24],NA)
    heartattack <- ifelse(is.na(x[,25])==0 & x[,25] %in% c(0,1),x[,25],NA)
    heartdisease <- ifelse(is.na(x[,26])==0 & x[,26] %in% c(0,1),x[,26],NA)
    stroke <- ifelse(is.na(x[,27])==0 & x[,27] %in% c(0,1),x[,27],NA)
    diab <- ifelse(is.na(x[,28])==0 & x[,28] %in% c(0,1),x[,28],NA)
    bron <- ifelse(is.na(x[,29])==0 & x[,29] %in% c(0,1),x[,29],NA)
    kidney <- ifelse(is.na(x[,30])==0 & x[,30] %in% c(0,1),x[,30],NA)
    liver <- ifelse(is.na(x[,31])==0 & x[,31] %in% c(0,1),x[,31],NA)
    speceq <- ifelse(is.na(x[,32])==0 & x[,32] %in% c(0,1),x[,32],NA)
    year <- ifelse(is.na(x[,33])==0 & x[,33]>=0,x[,33],NA)
  }

  if (impute.missing==TRUE){
       lp <- -10.3915+1.6949*log(age[is.na(speceq)==1])+0.6288*I(race[is.na(speceq)==1]==1)+
             0.1398*I(race[is.na(speceq)==1]==2)+0.0565*I(race[is.na(speceq)==1]==3)+0.1878*female[is.na(speceq)==1]-
             0.1024*sqrt(cpd[is.na(speceq)==1])+0.015*pkyr.cat[is.na(speceq)==1]+0.2012*sqrt(smkyears[is.na(speceq)==1])+
             0.0178*qtyears[is.na(speceq)==1]
       lp <- checkImputed(lp, "health problem requiring special equipment", warn=1)
       speceq[is.na(speceq)==1] <- exp(lp)/(1+exp(lp))
       lp <- -3.0702-0.029*age[is.na(liver)==1]-0.3303*I(race[is.na(liver)==1]==1)+0.0984*I(race[is.na(liver)==1]==2)-
             0.606*I(race[is.na(liver)==1]==3)-0.0058*female[is.na(liver)==1]+0.1473*sqrt(cpd[is.na(liver)==1])-
             0.2633*log(pkyr.cat[is.na(liver)==1])+0.0434*smkyears[is.na(liver)==1]+0.115*sqrt(qtyears[is.na(liver)==1])
       lp <- checkImputed(lp, "liver condition in past year", warn=1)
       liver[is.na(liver)==1] <- exp(lp)/(1+exp(lp))
       lp <- -10.5384+1.8572*log(age[is.na(diab)==1])+0.7984*I(race[is.na(diab)==1]==1)+0.7837*I(race[is.na(diab)==1]==2)+
             0.5*I(race[is.na(diab)==1]==3)-0.2197*female[is.na(diab)==1]+0.1815*I(cpd[is.na(diab)==1] >= 20)+
             0.007*pkyr.cat[is.na(diab)==1]+0.2594*log(smkyears[is.na(diab)==1])+0.0076*qtyears[is.na(diab)==1]
       lp <- checkImputed(lp, "diabetes", warn=1)
       diab[is.na(diab)==1] <- exp(lp)/(1+exp(lp))
       lp <- -8.3008+0.0349*age[is.na(kidney)==1]+0.343*I(race[is.na(kidney)==1]==1)+0.1427*I(race[is.na(kidney)==1]==2)+
             0.3635*I(race[is.na(kidney)==1]==3)-0.2092*female[is.na(kidney)==1]+0.0281*cpd[is.na(kidney)==1]-
             0.2223*log(pkyr.cat[is.na(kidney)==1])+0.8292*log(smkyears[is.na(kidney)==1])+0.0176*qtyears[is.na(kidney)==1]
       lp <- checkImputed(lp, "weak/failing kidneys in past year", warn=1)
       kidney[is.na(kidney)==1] <- exp(lp)/(1+exp(lp))
       lp <- -17.818+3.6777*log(age[is.na(prior.cancer)==1])-0.5575*I(race[is.na(prior.cancer)==1]==1)-
             0.8102*I(race[is.na(prior.cancer)==1]==2)-1.164*I(race[is.na(prior.cancer)==1]==3)+0.2055*female[is.na(prior.cancer)==1]+
             0.016*cpd[is.na(prior.cancer)==1]-0.0053*pkyr.cat[is.na(prior.cancer)==1]+0.1098*sqrt(smkyears[is.na(prior.cancer)==1])+
             0.0877*sqrt(qtyears[is.na(prior.cancer)==1])
       lp <- checkImputed(lp, "prior history of cancer", warn=1)
       prior.cancer[is.na(prior.cancer)==1] <- exp(lp)/(1+exp(lp))
       lp <- -7.0694+0.0596*age[is.na(stroke)==1]+0.3312*I(race[is.na(stroke)==1]==1)+0.2439*I(race[is.na(stroke)==1]==2)-
             0.3382*I(race[is.na(stroke)==1]==3)-0.2466*female[is.na(stroke)==1]-0.3095*I(cpd[is.na(stroke)==1] >= 20)+
             0.0166*pkyr.cat[is.na(stroke)==1]+0.0828*log(smkyears[is.na(stroke)==1])-0.0759*log(qtyears[is.na(stroke)==1]+1)
       lp <- checkImputed(lp, "stroke", warn=1)
       stroke[is.na(stroke)==1] <- exp(lp)/(1+exp(lp))
       lp <- -14.5767+2.7591*log(age[is.na(chd)==1])+0.1135*I(race[is.na(chd)==1]==1)+0.1116*I(race[is.na(chd)==1]==2)-
             0.0271*I(race[is.na(chd)==1]==3)-0.6922*female[is.na(chd)==1]-0.1556*sqrt(cpd[is.na(chd)==1])+
             0.249*sqrt(pkyr.cat[is.na(chd)==1])+0.0112*smkyears[is.na(chd)==1]+0.0202*qtyears[is.na(chd)==1]
       lp <- checkImputed(lp, "coronary heart disease", warn=1)
       chd[is.na(chd)==1] <- exp(lp)/(1+exp(lp))
       lp <- -17.9734+3.5297*log(age[is.na(heartattack)==1])+0.0232*I(race[is.na(heartattack)==1]==1)-
             0.1583*I(race[is.na(heartattack)==1]==2)+0.1133*I(race[is.na(heartattack)==1]==3)-0.6572*female[is.na(heartattack)==1]-
             0.0056*cpd[is.na(heartattack)==1]+0.1762*sqrt(pkyr.cat[is.na(heartattack)==1])+0.1129*log(smkyears[is.na(heartattack)==1])-
             0.0003*qtyears[is.na(heartattack)==1]
       lp <- checkImputed(lp, "heart attack", warn=1)
       heartattack[is.na(heartattack)==1] <- exp(lp)/(1+exp(lp))
       lp <- -8.2199+1.1237*log(age[is.na(heartdisease)==1])-0.1497*I(race[is.na(heartdisease)==1]==1)-
             0.592*I(race[is.na(heartdisease)==1]==2)+0.081*I(race[is.na(heartdisease)==1]==3)-0.0199*female[is.na(heartdisease)==1]+
             0.221*sqrt(cpd[is.na(heartdisease)==1])-0.232*log(pkyr.cat[is.na(heartdisease)==1])+0.0362*smkyears[is.na(heartdisease)==1]+
             0.1547*sqrt(qtyears[is.na(heartdisease)==1])
       lp <- checkImputed(lp, "other heart disease", warn=1)
       heartdisease[is.na(heartdisease)==1] <- exp(lp)/(1+exp(lp))
       lp <- -3.4777-0.0145*age[is.na(bron)==1]+0.2342*I(race[is.na(bron)==1]==1)-0.4939*I(race[is.na(bron)==1]==2)-
             0.8404*I(race[is.na(bron)==1]==3)+0.8134*female[is.na(bron)==1]-0.085*sqrt(cpd[is.na(bron)==1])+
             0.0199*pkyr.cat[is.na(bron)==1]+0.0315*smkyears[is.na(bron)==1]+0.0097*qtyears[is.na(bron)==1]
       lp <- checkImputed(lp, "chronic bronchitis in past year", warn=1)
       bron[is.na(bron)==1] <- exp(lp)/(1+exp(lp))
       lp <- -12.5734+2.8198*log(age[is.na(hypertension)==1])+0.9723*I(race[is.na(hypertension)==1]==1)+
            0.0862*I(race[is.na(hypertension)==1]==2)-0.0721*I(race[is.na(hypertension)==1]==3)-0.2521*female[is.na(hypertension)==1]+
            0.1108*log(cpd[is.na(hypertension)==1])-0.0019*pkyr.cat[is.na(hypertension)==1]+0.2135*log(smkyears[is.na(hypertension)==1])+
            0.0512*sqrt(qtyears[is.na(hypertension)==1])
       lp <- checkImputed(lp, "hypertension", warn=1)
       hypertension[is.na(hypertension)==1] <- exp(lp)/(1+exp(lp))
       lp <- -10.1981+1.5728*log(age[is.na(angina)==1])-0.0783*I(race[is.na(angina)==1]==1)-0.1044*I(race[is.na(angina)==1]==2)-
             0.8567*I(race[is.na(angina)==1]==3)-0.3057*female[is.na(angina)==1]-0.3474*log(cpd[is.na(angina)==1])+
             0.3091*sqrt(pkyr.cat[is.na(angina)==1])-0.0012*smkyears[is.na(angina)==1]+0.0172*qtyears[is.na(angina)==1]
       lp <- checkImputed(lp, "angina pectoris", warn=1)
       angina[is.na(angina)==1] <- exp(lp)/(1+exp(lp))
   }

  if (flag1){    
    covar <- data.frame(age=age,
                        bmi=bmi,
                        cpd=cpd,
                        emp=emp,
                        fam.lung.trend=fam.lung.trend,
                        female=female,
                        qtyears=qtyears,
                        smkyears=smkyears,
                        race=race,
                        edu6=edu6,
                        pkyr.cat=pkyr.cat,
                        asb=asb,
                        smoke.age.start=smoke.age.start,
                        pneu = pneu,
                        prior.cancer = prior.cancer,
                        fam.cancer.onset = fam.cancer.onset,
                        copd=copd,
                        dust=dust,
                        fam.cancer=fam.cancer,
                        fam.smoke.cancer=fam.smoke.cancer,
                        no.hayfever=no.hayfever,
                        age.stopped=age.stopped,
                        asian = asian,
                        islander = islander,
                        indian = indian,
                        hypertension = hypertension,
                        chd = chd,
                        angina = angina,
                        heartattack = heartattack,
                        heartdisease = heartdisease,
                        stroke = stroke,
                        diab = diab,
                        bron = bron,
                        kidney = kidney,
                        liver = liver,
                        speceq = speceq,
                        year = year)
  } else {
    covar <- data.frame(age=age,
                        bmi=bmi,
                        cpd=cpd,
                        emp=emp,
                        fam.lung.trend=fam.lung.trend,
                        female=female,
                        qtyears=qtyears,
                        smkyears=smkyears,
                        race=race,
                        edu6=edu6,
                        pkyr.cat=pkyr.cat,
                        asb=asb,
                        smoke.age.start=smoke.age.start,
                        pneu = pneu,
                        prior.cancer = prior.cancer,
                        fam.cancer.onset = fam.cancer.onset,
                        copd=copd,
                        dust=dust,
                        fam.cancer=fam.cancer,
                        fam.smoke.cancer=fam.smoke.cancer,
                        no.hayfever=no.hayfever,
                        age.stopped=age.stopped,
                        asian = asian,
                        islander = islander,
                        indian = indian)
  }

  # Incidence calculations use the original race
  covar$race <- race.orig

   ### Bach 2003 ###   
   r.bach <- risk.bach.10(covar, nyears.bach)
   ### Hoggart 2012 ###
   r.hoggart <- risk.hoggart(covar)
   ### LLP 2008 ###
   r.llp <- try(risk.llp(covar), silent=TRUE)
   if ("try-error" %in% class(r.llp)) r.llp <- NA
   ### LLPi 2015 ###
   r.llpi <- risk.llpi(covar)
   ### Spitz 2007 ###
   r.spitz <- risk.spitz(covar)
   ### Tammemagi 2013 ###
   r.tamme <- risk.tammemagi(covar)
   ### Pittsburgh 2015 ###
   r.pitt <- risk.pittsburgh(covar)
      
   # Changed below 2021-07-16, from email: 
   # 2)	The USPSTF recently changed their criteria to age 50-80, 20+ pack-years, and no more than 15 years quit
   # uspstf_elig <- ifelse(age>=55 & age<=80 & pkyr.cat>=30 & qtyears <= 15,1,0)
   uspstf_elig <- ifelse(age>=50 & age<=80 & pkyr.cat>=20 & qtyears <= 15,1,0)

   # calculate predicted risks of lung cancer death within 5 based on LCDRAT with competing risk of death
   covar$LCDRAT            <- risk.kovalchik(0, 5, covar[,1:11], LCDRAT, cox.death)
   covar$constrainedLCDRAT <- risk.kovalchik(0, 5, covar[,1:11], constrained.LCDRAT, constrained.cox.death)
   
   #calculate predicted risks of lung cancer death using chest xray
   covar$cxLCDRAT <- 0.796*covar$LCDRAT
   
   #calculate predicted risks of lung cancer incidence within 5 years based on LCRAT with competing risk of death
   covar$LCRAT            <- risk.kovalchik(0, 5, covar[,1:11], LCRAT, cox.death)
   covar$LCRAT            <- pmax(covar$LCRAT,covar$LCDRAT)
   covar$constrainedLCRAT <- risk.kovalchik(0, 5, covar[,1:11], constrained.LCRAT, constrained.cox.death)
   covar$constrainedLCRAT <- pmax(covar$constrainedLCRAT,covar$constrainedLCDRAT)
      
   #calculate predicted risks of lung cancer incidence using chest xray
   covar$cxLCRAT <- 1.124*covar$LCRAT

   #calculate probability of Y, number of false positives, taking values of 0, 1, 2, or 3 
   #responses are reported as log(P(Y=1)/P(Y=0)), log(P(Y=2)/P(Y=0)), and log(P(Y=3)/P(Y=0))
   prob_numfalsepos <- try(predict(polytmod,type="response",newdata=covar), silent=TRUE)
   if ("try-error" %in% class(prob_numfalsepos)) prob_numfalsepos <- matrix(data=NA, nrow=1, ncol=4)

   #solving for P(Y=0), P(Y=1), P(Y=2), and P(Y=3), we have:
   covar$prob_0falsepos <- ifelse(is.na(covar$cxLCRAT),NA,prob_numfalsepos[,1])
   covar$prob_1falsepos <- ifelse(is.na(covar$cxLCRAT),NA,prob_numfalsepos[,2])
   covar$prob_2falsepos <- ifelse(is.na(covar$cxLCRAT),NA,prob_numfalsepos[,3])
   covar$prob_3falsepos <- ifelse(is.na(covar$cxLCRAT),NA,prob_numfalsepos[,4])
   covar$expected_falsepos <- covar$prob_1falsepos + 2*covar$prob_2falsepos + 3*covar$prob_3falsepos
   
   covar$LCDRAT1 <- risk.kovalchik(0, 1, covar[,1:11], LCDRAT, cox.death)
   covar$LCDRAT2 <- risk.kovalchik(0, 2, covar[,1:11], LCDRAT, cox.death)
   covar$LCDRAT3 <- risk.kovalchik(0, 3, covar[,1:11], LCDRAT, cox.death)
   covar$LCDRAT4 <- risk.kovalchik(0, 4, covar[,1:11], LCDRAT, cox.death)
   covar$LCDRAT5 <- risk.kovalchik(0, 5, covar[,1:11], LCDRAT, cox.death)
   
   # Life expectancy and time gained use the counterfactual race
   covar$race <- race.counter

   # Maximum age for overall mortality calculation
   maxAge.mortality <- 98

   if (flag1) {   
     covar$missval <- ifelse(!is.na(covar$age) & !is.na(covar$female) & !is.na(covar$smkyears) & !is.na(covar$qtyears) &
                             !is.na(covar$cpd) & !is.na(covar$race) & !is.na(covar$emp) & 
                             !is.na(covar$fam.lung.trend) & !is.na(covar$bmi) & !is.na(covar$edu6) &
                             !is.na(covar$year) & !is.na(covar$hypertension) & !is.na(covar$chd) &
                             !is.na(covar$angina) & !is.na(covar$stroke) & !is.na(covar$heartattack) &
                             !is.na(covar$heartdisease) & !is.na(covar$prior.cancer) & !is.na(covar$speceq) &
                             !is.na(covar$diab) & !is.na(covar$liver) & !is.na(covar$bron) & !is.na(covar$kidney),0,1)

     lgindex         <- which(covar$missval==0)
     covar$ely       <- NA
     covar$lg        <- NA
     covar$ely_ct    <- NA
     covar$mortality <- NA

     rs <- predict(morat, covar, type = "risk",reference="sample")
     if (sum(1-covar$missval)>0){
       z <- covar[lgindex,]

       # Warning for all-cause mortality if age is too large  
       if (!counterFlag && any(z[, 1, drop=TRUE] + nyears.mortality > maxAge.mortality)) {
         msg <- paste0("The all-cause mortality risk was computed using a maximum age of ",
                        maxAge.mortality)
         warning(msg)
       }

       lyg <- function(x){
         #print(x)
         #agex <- age[x]
         agex <- z[x, 1]
         if ((agex < 40) || (agex >= 99)) {
           msg <- paste0("Age = ", agex, " is outside the range for estimation, results may not be accurate.")
           warning(msg)
         }
         numer <- exp(-1*haz[which(haz$time>agex),1])
         ival  <- getMinLogVecLE(haz$time, agex)
         if (is.finite(ival)) {
           denom <- exp(-1*haz[ival,1])
         } else {
           denom <- NA
         }         
         if (agex > 40) {
           survx <- (numer/denom)^rs[x]
         } else {
           survx <- numer
         }
         #survx <- (exp(-1*haz[which(haz$time>age[x]),1])/exp(-1*haz[which(haz$time==age[x]),1]))^rs[x]

         ages <- haz[which(haz$time>agex),2]
         ely <- sum(diff(c(z$age[x],ages))*survx)  
 
         agexp5 <- agex + 5
         numer  <- exp(-1*haz[which(haz$time>agexp5),1])
         ival   <- getMinLogVecLE(haz$time, agexp5)
         if (is.finite(ival)) {
           denom <- exp(-1*haz[ival,1])
         } else {
           denom <- NA
         }  
         condsurvx <- (numer/denom)^rs[x]
         #condsurvx <- (exp(-1*haz[which(haz$time>age[x]+5),1])/exp(-1*haz[which(haz$time==age[x]+5),1]))^rs[x]

         condages <- haz[which(haz$time>agexp5),2]           
         condely <- sum(diff(c(z$age[x]+5,condages))*condsurvx)
         lyg <- .204*z$LCDRAT1[x] + .204*z$LCDRAT2[x] + 
                .204*z$LCDRAT3[x] + .204*z$LCDRAT4[x] +
                .204*z$LCDRAT5[x]*(1+condely)

         #######################################################
         # For overall mortality in m years
         # Update agexp5 and numer, denom remains the same.
         agexp.mor <- agex + nyears.mortality
         if (agexp.mor > maxAge.mortality) agexp.mor <- maxAge.mortality
         numer.mor     <- exp(-1*haz[which(haz$time>agexp.mor),1])
         condsurvx.mor <- 1 - (numer.mor[1]/denom)^rs[x]
         if (is.finite(condsurvx.mor) && (condsurvx.mor < 0)) condsurvx.mor <- 0
         ####################################################### 

         res <- c(ely,ely+lyg,365.25*lyg, condsurvx.mor)
         return(res)
       } 
       lyg_res                  <- sapply(1:nrow(z),lyg)
       lyg_res                  <- t(lyg_res)
       covar$ely[lgindex]       <- lyg_res[,1]
       covar$ely_ct[lgindex]    <- lyg_res[,2]
       covar$lg[lgindex]        <- lyg_res[,3]
       covar$mortality[lgindex] <- lyg_res[,4]

       # lyg calculations based on 5 years
       covar$LYG1 <- (covar$lg/365.25)/covar$cxLCRAT
       covar$LYG2 <- (covar$lg/365.25)/(covar$LCDRAT - covar$cxLCDRAT)
     }
   
     #################################################################################
     # Redo some of the above calculations using nyears instead of 5 years.
     # The code was placed here because the lyg calculations must be based on 5 years.
     #################################################################################

     # calculate predicted risks of lung cancer death within nyears based on LCDRAT with competing risk of death
     covar$LCDRAT            <- risk.kovalchik(0, nyears, covar[,1:11], LCDRAT, cox.death)
     covar$constrainedLCDRAT <- risk.kovalchik(0, nyears, covar[,1:11], constrained.LCDRAT, constrained.cox.death)
   
     #calculate predicted risks of lung cancer death using chest xray
     covar$cxLCDRAT <- 0.796*covar$LCDRAT
   
     #calculate predicted risks of lung cancer incidence within nyears years based on LCRAT with competing risk of death
     covar$LCRAT            <- risk.kovalchik(0, nyears, covar[,1:11], LCRAT, cox.death)
     covar$LCRAT            <- pmax(covar$LCRAT,covar$LCDRAT)
     covar$constrainedLCRAT <- risk.kovalchik(0, nyears, covar[,1:11], constrained.LCRAT, constrained.cox.death)
     covar$constrainedLCRAT <- pmax(covar$constrainedLCRAT,covar$constrainedLCDRAT)
      
     #calculate predicted risks of lung cancer incidence using chest xray
     covar$cxLCRAT <- 1.124*covar$LCRAT

     #calculate probability of Y, number of false positives, taking values of 0, 1, 2, or 3 
     #responses are reported as log(P(Y=1)/P(Y=0)), log(P(Y=2)/P(Y=0)), and log(P(Y=3)/P(Y=0))
     prob_numfalsepos <- try(predict(polytmod,type="response",newdata=covar), silent=TRUE)
     if ("try-error" %in% class(prob_numfalsepos)) prob_numfalsepos <- matrix(data=NA, nrow=1, ncol=4)

     #solving for P(Y=0), P(Y=1), P(Y=2), and P(Y=3), we have:
     covar$prob_0falsepos    <- ifelse(is.na(covar$cxLCRAT),NA,prob_numfalsepos[,1])
     covar$prob_1falsepos    <- ifelse(is.na(covar$cxLCRAT),NA,prob_numfalsepos[,2])
     covar$prob_2falsepos    <- ifelse(is.na(covar$cxLCRAT),NA,prob_numfalsepos[,3])
     covar$prob_3falsepos    <- ifelse(is.na(covar$cxLCRAT),NA,prob_numfalsepos[,4])
     covar$expected_falsepos <- covar$prob_1falsepos + 2*covar$prob_2falsepos + 3*covar$prob_3falsepos
     #################################################################################

     predicted <- data.frame(uspstf_elig,
                             covar$LCDRAT,
                             (covar$LCDRAT - covar$cxLCDRAT),
                             covar$LCRAT,
                             (covar$cxLCRAT - covar$LCRAT),
                             (1-covar$prob_0falsepos),
                             covar$expected_falsepos,
                             r.bach,
                             r.hoggart,
                             r.llp,
                             r.llpi,
                             r.spitz,
                             r.tamme,
                             r.pitt,
                             covar$constrainedLCDRAT,
                             covar$constrainedLCRAT,
                             covar$lg,
                             covar$ely,
                             covar$ely_ct,
                             covar$LYG1,
                             covar$LYG2,
                             covar$mortality)
                           
     colnames(predicted) <- c(
       "USPSTF eligible",
       paste0("Risk of lung cancer death within ", nyears, " years without screening  (Katki, 2016)"),
       paste0("Reduction in ", nyears, "-years risk of lung cancer death with CT screening  (Katki, 2016)"),
       paste0("Risk of lung cancer diagnosis within ", nyears, " years without screening  (Katki, 2016)"),
       paste0("Increase in ", nyears, "-years risk lung cancer diagnosis with CT screening (Katki, 2016)"),
       "Probability of a false-positive CT lung screens (Katki, 2016)",
       "Expected number of false-positive CT lung screens (Katki, 2016)",
       paste0("Risk of lung cancer diagnosis within ", nyears.bach, " years (Bach, 2003)"),
       "Risk of lung cancer diagnosis within 1 year (Hoggart, 2012)",
       "Risk of lung cancer diagnosis within 5 years (LLP, 2008)",
       "Risk of lung cancer diagnosis within 8.7 years (LLPi, 2015)",
       "Risk of lung cancer diagnosis within 1 year (Spitz, 2007)",
       "Risk of lung cancer diagnosis within 6 years (Tammemagi, 2013)",
       "Risk of lung cancer diagnosis within 6 years (Pittsburgh, 2015)",
       paste0("Risk of lung cancer death within ", nyears, " years (constrained version of LCDRAT)"),
       paste0("Risk of lung cancer diagnosis within ", nyears, " years (constrained version of LCRAT)"),
       "LYFS-CT: Days of life gained from undergoing 3 rounds of CT lung screens (Cheung, 2019)",
       "Life-expectancy without CT lung screens",
       "Life-expectancy with 3 rounds of CT lung screens",
       "Years of life gained if lung cancer is found early due to screening",
       "Years of life gained if lung cancer death is averted due to screening",
       paste0("All-cause mortality risk in ", nyears.mortality, " years"))
   } else{
     predicted <- data.frame(uspstf_elig,
                             covar$LCDRAT,
                             (covar$LCDRAT - covar$cxLCDRAT),
                             covar$LCRAT,
                             (covar$cxLCRAT - covar$LCRAT),
                             (1-covar$prob_0falsepos),
                             covar$expected_falsepos,
                             r.bach,
                             r.hoggart,
                             r.llp,
                             r.llpi,
                             r.spitz,
                             r.tamme,
                             r.pitt,
                             covar$constrainedLCDRAT,
                             covar$constrainedLCRAT)
     
     colnames(predicted) <- c(
       "USPSTF eligible",
       paste0("Risk of lung cancer death within ", nyears, " years without screening  (Katki, 2016)"),
       paste0("Reduction in ", nyears, "-years risk of lung cancer death with CT screening  (Katki, 2016)"),
       paste0("Risk of lung cancer diagnosis within ", nyears, " years without screening  (Katki, 2016)"),
       paste0("Increase in ", nyears, "-years risk lung cancer diagnosis with CT screening (Katki, 2016)"),
       "Probability of a false-positive CT lung screens (Katki, 2016)",
       "Expected number of false-positive CT lung screens (Katki, 2016)",
       paste0("Risk of lung cancer diagnosis within ", nyears.bach, " years (Bach, 2003)"),
       "Risk of lung cancer diagnosis within 1 year (Hoggart, 2012)",
       "Risk of lung cancer diagnosis within 5 years (LLP, 2008)",
       "Risk of lung cancer diagnosis within 8.7 years (LLPi, 2015)",
       "Risk of lung cancer diagnosis within 1 year (Spitz, 2007)",
       "Risk of lung cancer diagnosis within 6 years (Tammemagi, 2013)",
       "Risk of lung cancer diagnosis within 6 years (Pittsburgh, 2015)",
       paste0("Risk of lung cancer death within ", nyears, " years (constrained version of LCDRAT)"),
       paste0("Risk of lung cancer diagnosis within ", nyears, " years (constrained version of LCRAT)"))
   }
   
   predicted                         
}
